package net.wanji.utc.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.Result;
import net.wanji.utc.common.exception.ControlException;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.po.CrossInfoPO;
import net.wanji.utc.service.ControlCommandService;
import net.wanji.utc.util.ListUtil;
import net.wanji.utc.vo.PhaseLockVO;
import net.wanji.utc.vo.plansend.PlanSendVO;
import net.wanji.utc.vo.timeplan.TimePlanVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import java.util.List;

import static net.wanji.utc.common.ResultEnum.BODY_NOT_MATCH;

/**
 * 控制指令接口
 *
 * @author wuxiaokai
 * @date 2022/11/15 13:21:10
 */
@Slf4j
@RestController
@RequestMapping("/controlCommand")
@Api(value = "控制指令接口", description = "控制指令接口", tags = "控制指令接口")
public class ControlCommandController {

    @Value("${signal.mock}")
    private boolean mock;

    @Resource
    private CrossInfoMapper crossInfoMapper;

    @Autowired
    private ControlCommandService hkControlCommandService;

    /**
     *
     */
    @ApiOperation(value = "方案下发-基础方案下发", notes = "方案下发-基础方案下发")
    @PostMapping("/schemeSend")
    public <T> Result<T> schemeSend(@RequestParam String signalId, @RequestParam Integer command) {
        if (mock) {
            return Result.success();
        }
        Integer manufacturerId = check(signalId);
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getId())) {
            return hkControlCommandService.allRedControl(signalId, null, null, command);
        } else {
            // todo else
            return null;
        }
    }

    /**
     * 相位配时下发
     *
     * @param signalId  信号id
     * @param timePlans 配时方案实体类
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "相位配时下发-相位的绿灯、绿闪、行闪、黄灯、红灯配时下发", notes = "相位配时下发-相位的绿灯、绿闪、行闪、黄灯、红灯配时下发")
    @PostMapping("/phaseTimingSend")
    public <T> Result<T> phaseTimingSend(@RequestParam String signalId, @RequestBody List<TimePlanVO> timePlans) {
        if (mock) {
            return Result.success();
        }
        CrossInfoPO crossInfoPo = crossInfoMapper.selectByCode(signalId);
        if (crossInfoPo == null) {
            throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，信号机ID不正确。");
        }
        if (ListUtil.isEmpty(timePlans)) {
            throw new ControlException("参数错误，下发方案信息为空。");
        }
        //开启适配服务
        for (TimePlanVO timePlanVo : timePlans) {
            timePlanVo.setCode(crossInfoPo.getCode());
            timePlanVo.setManufacturerCode(crossInfoPo.getManufacturerId() + "");
        }
        Integer manufacturerId = crossInfoPo.getManufacturerId();
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getId())) {
            return hkControlCommandService.phaseTimingSend(timePlans);
        } else {
            // todo else
            return null;
        }
    }

    /**
     * 计划下发
     *
     * @param signalId   信号id
     * @param planSendVO 计划下发VO
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "计划下发-计划信息、时段信息", notes = "计划下发-计划信息、时段信息")
    @PostMapping("/planSend")
    public <T> Result<T> planSend(@RequestParam String signalId, @RequestBody PlanSendVO planSendVO) {
        if (mock) {
            return Result.success();
        }
        try {
            CrossInfoPO crossInfoPo = crossInfoMapper.selectByCode(signalId);
            if (crossInfoPo == null) {
                throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，信号机ID不正确。");
            }
            planSendVO.setCode(crossInfoPo.getCode());
            planSendVO.setManufacturerCode(crossInfoPo.getManufacturerId() + "");
            Integer manufacturerId = crossInfoPo.getManufacturerId();
            if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getId())) {
                return hkControlCommandService.planSend(planSendVO);
            } else {

                // todo else
                return null;
            }
        } catch (Exception ex) {
            log.error("计划下发-计划信息、时段信息异常：{}", ex.getMessage());
            throw new ControlException("计划下发-计划信息、时段信息异常");
        }
    }

    /**
     * 全红控制
     *
     * @param signalId 信号机id
     * @param command  指令
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "全红控制-路口全红控制/恢复", notes = "全红控制-路口全红控制/恢复")
    @PostMapping("/allRedControl")
    public <T> Result<T> allRedControl(@RequestParam String signalId, @RequestParam Integer command) {
        if (mock) {
            return Result.success();
        }
        Integer manufacturerId = check(signalId);
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getId())) {
            return hkControlCommandService.allRedControl(signalId, null, null, command);
        } else {
            // todo else
            return null;
        }
    }

    /**
     * 黄闪控制
     *
     * @param signalId 信号id
     * @param command  命令
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "黄闪控制-路口黄闪控制/恢复", notes = "黄闪控制-路口黄闪控制/恢复")
    @PostMapping("/yellowLightControl")
    public <T> Result<T> yellowLightControl(@RequestParam String signalId, @RequestParam Integer command) {
        if (mock) {
            return Result.success();
        }
        Integer manufacturerId = check(signalId);
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getId())) {
            return hkControlCommandService.yellowLightControl(signalId, null, null, command);
        } else {
            // todo else
            return null;
        }
    }

    /**
     * 关灯控制
     *
     * @param signalId 信号id
     * @param command  命令
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "关灯控制-路口关灯控制/开灯", notes = "关灯控制-路口关灯控制/开灯")
    @PostMapping("/closeLightControl")
    public <T> Result<T> closeLightControl(@RequestParam String signalId, @RequestParam Integer command) {
        if (mock) {
            return Result.success();
        }
        Integer manufacturerId = check(signalId);
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getId())) {
            return hkControlCommandService.closeLightControl(signalId, null, null, command);
        } else {
            // todo else
            return null;
        }
    }

    /**
     * 步进控制
     *
     * @param signalId 信号id
     * @param command  命令
     * @param stepNum  一步num
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "步进控制-步进控制/恢复", notes = "步进控制-步进控制/恢复")
    @PostMapping("/stepControl")
    public <T> Result<T> stepControl(@RequestParam String signalId,
                                     @RequestParam Integer command,
                                     @RequestParam int stepNum) {
        if (mock) {
            return Result.success();
        }
        Integer manufacturerId = check(signalId);
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getId())) {
            return hkControlCommandService.stepControl(signalId, null, null, command, stepNum);
        } else {
            // todo else
            return null;
        }
    }

    /**
     * 恢复时间表
     *
     * @param signalId 信号id
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "恢复时间表-恢复路口时间表执行", notes = "恢复时间表-恢复路口时间表执行")
    @PostMapping("/scheduleSend")
    public <T> Result<T> scheduleSend(@RequestParam String signalId) {
        if (mock) {
            return Result.success();
        }
        Integer manufacturerId = check(signalId);
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getId())) {
            return hkControlCommandService.scheduleSend(signalId, null, null, null);
        } else {
            // todo else
            return null;
        }
    }

    /**
     * 锁定控制
     *
     * @param signalId 信号id
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "锁定控制-相位锁定/解锁", notes = "锁定控制-相位锁定/解锁")
    @PostMapping("/lockControl")
    public <T> Result<T> lockControl(@RequestParam String signalId, @RequestBody PhaseLockVO phaseLock) {
        if (mock) {
            return Result.success();
        }
        try {
            Integer manufacturerId = check(signalId);
            if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getId())) {
                return hkControlCommandService.lockControl(signalId, phaseLock);
            } else {
                // todo else
                return null;
            }
        } catch (Exception ex) {
            log.error("锁定控制-相位锁定/解锁异常：{}", ex.getMessage());
            throw new ControlException("锁定控制-相位锁定/解锁异常");
        }
    }

    private Integer check(String signalId) {
        CrossInfoPO crossInfoPo = crossInfoMapper.selectByCode(signalId);
        if (crossInfoPo == null) {
            throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，信号机ID不正确。");
        }
        return crossInfoPo.getManufacturerId();
    }
}
